@@ -3,6 +3,7 @@
 #include "vtkDICOMMetaData.h"
 #include "vtkDICOMSorter.h"
 #include "vtkDICOMReader.h"
+#include "vtkDICOMCTRectifier.h"
 
 #include "vtkRenderWindowInteractor.h"
 #include "vtkInteractorStyleImage.h"
@@ -10,6 +11,7 @@
 #include "vtkRenderer.h"
 #include "vtkCamera.h"
 #include "vtkImageData.h"
+#include "vtkImageReslice.h"
 #include "vtkImageSliceMapper.h"
 #include "vtkImageProperty.h"
 #include "vtkImageSlice.h"
@@ -17,6 +19,8 @@
 #include "vtkSmartPointer.h"
 #include "vtkStringArray.h"
 #include "vtkIntArray.h"
+#include "vtkMatrix4x4.h"
+#include "vtkMath.h"
 
 int main(int argc, char *argv[])
 {
@@ -36,9 +40,21 @@ int main(int argc, char *argv[])
   vtkSmartPointer<vtkStringArray> files =
     vtkSmartPointer<vtkStringArray>::New();
 
+  const char *stackID = 0;
+
   for (int i = 1; i < argc; i++)
     {
-    files->InsertNextValue(argv[i]);
+    if (strcmp(argv[i], "--stack") == 0)
+      {
+      if (i+1 < argc)
+        {
+        stackID = argv[++i];
+        }
+      }
+    else
+      {
+      files->InsertNextValue(argv[i]);
+      }
     }
 
   sorter->SetInputFileNames(files);
@@ -72,12 +88,17 @@ int main(int argc, char *argv[])
 
   // trust the user and display all the files, even if multiple series
   vtkStringArray *a = sorter->GetOutputFileNames();
+  vtkSmartPointer<vtkImageReslice> reslice =
+    vtkSmartPointer<vtkImageReslice>::New();
   vtkSmartPointer<vtkDICOMReader> reader =
     vtkSmartPointer<vtkDICOMReader>::New();
   reader->SetMemoryRowOrderToFileNative();
   //reader->TimeAsVectorOn();
   //reader->SetDesiredTimeIndex(5);
-  //reader->SetDesiredStackID("1");
+  if (stackID)
+    {
+    reader->SetDesiredStackID(stackID);
+    }
   reader->SetFileNames(a);
 
   double range[2];
@@ -86,6 +107,71 @@ int main(int argc, char *argv[])
   reader->GetOutput()->GetScalarRange(range);
   reader->GetOutput()->GetExtent(extent);
 
+  // get the output port to connect to the display pipeline
+  vtkAlgorithmOutput *portToDisplay = reader->GetOutputPort();
+
+#if 0
+  // check if the matrix indicates a tilted CT gantry
+  vtkSmartPointer<vtkMatrix4x4> pmat =
+    vtkSmartPointer<vtkMatrix4x4>::New();
+  pmat->DeepCopy(reader->GetPatientMatrix());
+  double xvec[4] = { 1.0, 0.0, 0.0, 0.0 };
+  double yvec[4] = { 0.0, 1.0, 0.0, 0.0 };
+  double zvec[4] = { 0.0, 0.0, 1.0, 0.0 };
+  pmat->MultiplyPoint(xvec, xvec);
+  pmat->MultiplyPoint(yvec, yvec);
+  pmat->MultiplyPoint(zvec, zvec);
+
+  // create slice normal and compare to zvec
+  double normal[3];
+  vtkMath::Cross(xvec, yvec, normal);
+  double checkvec[3];
+  vtkMath::Cross(zvec, normal, checkvec);
+  if (vtkMath::Norm(checkvec) > 1e-3)
+    {
+    // definite gantry tilt, the volume is trapezoid-shaped
+    vtkSmartPointer<vtkMatrix4x4> rmat =
+      vtkSmartPointer<vtkMatrix4x4>::New();
+    rmat->DeepCopy(pmat);
+    rmat->SetElement(0, 2, normal[0]);
+    rmat->SetElement(1, 2, normal[1]);
+    rmat->SetElement(2, 2, normal[2]);
+    pmat->Invert();
+    vtkMatrix4x4::Multiply4x4(pmat, rmat, rmat);
+
+    // pure shear matrix will have only one element that is different
+    // from the identity matrix:
+    double shear = rmat->GetElement(1, 2);
+    rmat->Identity();
+    rmat->SetElement(1, 2, shear);
+
+    double origin[3], spacing[3];
+    reader->GetOutput()->GetOrigin(origin);
+    reader->GetOutput()->GetSpacing(spacing);
+    // adjust the origin to centre the new volume on the old trapezoid
+    origin[1] -= shear*0.5*spacing[2]*(extent[5] - extent[4]);
+
+    // use vtkImageReslice to eliminate any shear
+    reslice->SetOutputOrigin(origin);
+    reslice->SetOutputSpacing(spacing);
+    reslice->SetOutputExtent(extent);
+    reslice->SetResliceAxes(rmat);
+    reslice->SetInputConnection(reader->GetOutputPort());
+    reslice->SetInterpolationModeToLinear();
+    reslice->Update();
+
+    // specify the port that the display pipeline will use
+    portToDisplay = reslice->GetOutputPort();
+    }
+#else
+  vtkSmartPointer<vtkDICOMCTRectifier> rect =
+    vtkSmartPointer<vtkDICOMCTRectifier>::New();
+  rect->SetVolumeMatrix(reader->GetPatientMatrix());
+  rect->SetInputConnection(reader->GetOutputPort());
+  rect->Update();
+  portToDisplay = rect->GetOutputPort();
+#endif
+
   static double viewport[3][4] = {
     { 0.67, 0.0, 1.0, 0.5 },
     { 0.67, 0.5, 1.0, 1.0 },
@@ -101,7 +187,7 @@ int main(int argc, char *argv[])
       vtkSmartPointer<vtkImageSliceMapper>::New();
     if (i < 3)
       {
-      imageMapper->SetInputConnection(reader->GetOutputPort());
+      imageMapper->SetInputConnection(portToDisplay);
       }
     imageMapper->SetOrientation(i % 3);
     imageMapper->SliceAtFocalPointOn();
@@ -168,7 +254,7 @@ int main(int argc, char *argv[])
   vtkStringArray *sarray = reader->GetStackIDs();
   if (sarray->GetNumberOfValues())
     {
-    cout << "StackIDs:";
+    cout << "StackIDs (choose one with --stack):";
     for (vtkIdType ii = 0; ii < sarray->GetNumberOfValues(); ii++)
       {
       cout << " \"" << sarray->GetValue(ii) << "\"";
