@@ -17,6 +17,7 @@
 #include "vtkDICOMMetaData.h"
 #include "vtkDICOMParser.h"
 #include "vtkDICOMUtilities.h"
+#include "vtkDICOMVR.h"
 
 #include <vtkObjectFactory.h>
 #include <vtkSmartPointer.h>
@@ -60,8 +61,7 @@ struct vtkDICOMDirectory::StudyItem
 struct vtkDICOMDirectory::PatientItem
 {
   vtkDICOMItem Record;
-  int FirstStudy;
-  int LastStudy;
+  vtkSmartPointer<vtkIntArray> Studies;
 };
 
 class vtkDICOMDirectory::SeriesVector
@@ -127,6 +127,7 @@ vtkDICOMDirectory::vtkDICOMDirectory()
   this->InternalFileName = 0;
   this->RequirePixelData = 1;
   this->ScanDepth = 1;
+  this->Query = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -145,6 +146,7 @@ vtkDICOMDirectory::~vtkDICOMDirectory()
   delete this->Studies;
   delete this->Patients;
   delete [] this->FileSetID;
+  delete this->Query;
 }
 
 //----------------------------------------------------------------------------
@@ -189,6 +191,21 @@ void vtkDICOMDirectory::SetDirectoryName(const char *name)
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
+void vtkDICOMDirectory::SetFindQuery(const vtkDICOMItem& item)
+{
+  if (this->Query != &item)
+    {
+    delete this->Query;
+    this->Query = 0;
+    if (!item.IsEmpty())
+      {
+      this->Query = new vtkDICOMItem;
+      *(this->Query) = item;
+      }
+    }
+}
+
 //----------------------------------------------------------------------------
 int vtkDICOMDirectory::GetNumberOfSeries()
 {
@@ -244,15 +261,9 @@ const vtkDICOMItem& vtkDICOMDirectory::GetPatientRecord(int patient)
 }
 
 //----------------------------------------------------------------------------
-int vtkDICOMDirectory::GetFirstStudyForPatient(int patient)
+vtkIntArray *vtkDICOMDirectory::GetStudiesForPatient(int patient)
 {
-  return (*this->Patients)[patient].FirstStudy;
-}
-
-//----------------------------------------------------------------------------
-int vtkDICOMDirectory::GetLastStudyForPatient(int patient)
-{
-  return (*this->Patients)[patient].LastStudy;
+  return (*this->Patients)[patient].Studies;
 }
 
 //----------------------------------------------------------------------------
@@ -296,12 +307,25 @@ void vtkDICOMDirectory::AddSeriesFileNames(
     this->Patients->push_back(PatientItem());
     PatientItem& item = this->Patients->back();
     item.Record = patientRecord;
-    item.FirstStudy = study;
-    item.LastStudy = study;
+    item.Studies = vtkSmartPointer<vtkIntArray>::New();
+    item.Studies->InsertNextValue(study);
     }
   else if (m >= 0 && patient == m-1)
     {
-    (*this->Patients)[patient].LastStudy = study;
+    PatientItem& item = (*this->Patients)[patient];
+    vtkIdType nn = item.Studies->GetMaxId() + 1;
+    vtkIdType ii = 0;
+    for (; ii < nn; ii++)
+      {
+      if (study == item.Studies->GetValue(ii))
+        {
+        break;
+        }
+      }
+    if (ii == nn)
+      {
+      item.Studies->InsertNextValue(study);
+      }
     }
   else
     {
@@ -388,21 +412,71 @@ void vtkDICOMDirectory::FillPatientRecord(
 //----------------------------------------------------------------------------
 void vtkDICOMDirectory::SortFiles(vtkStringArray *input)
 {
-  vtkSmartPointer<vtkUnsignedShortArray> groups =
-    vtkSmartPointer<vtkUnsignedShortArray>::New();
   vtkSmartPointer<vtkDICOMMetaData> meta =
     vtkSmartPointer<vtkDICOMMetaData>::New();
+  vtkSmartPointer<vtkDICOMMetaData> query =
+    vtkSmartPointer<vtkDICOMMetaData>::New();
   vtkSmartPointer<vtkDICOMParser> parser =
     vtkSmartPointer<vtkDICOMParser>::New();
 
   parser->AddObserver(
     vtkCommand::ErrorEvent, this, &vtkDICOMDirectory::RelayError);
 
-  groups->InsertNextValue(0x0008); // For study and series info.
-  groups->InsertNextValue(0x0010); // For patient info.
-  groups->InsertNextValue(0x0020); // For study and series info.
   parser->SetMetaData(meta);
-  parser->SetGroups(groups);
+
+  // these are the attributes that must be part of the query
+  static const DC::EnumType requiredElements[] = {
+    // basic required information
+    DC::SpecificCharacterSet,
+    // series-level information
+    DC::SeriesDate,
+    DC::SeriesTime,
+    DC::Modality,
+    DC::SeriesDescription,
+    DC::BodyPartExamined,
+    DC::SeriesInstanceUID,
+    DC::SeriesNumber,
+    // study-level information
+    DC::StudyDate,
+    DC::StudyTime,
+    DC::ReferringPhysicianName,
+    DC::PatientAge,
+    DC::StudyInstanceUID,
+    DC::StudyID,
+    DC::AccessionNumber,
+    DC::StudyDescription,
+    // patient-level information
+    DC::PatientName,
+    DC::PatientID,
+    DC::PatientBirthDate,
+    DC::PatientSex,
+    // delimiter to mark end of list
+    DC::ItemDelimitationItem
+  };
+
+  for (const DC::EnumType *tagPtr = requiredElements;
+       *tagPtr != DC::ItemDelimitationItem;
+       ++tagPtr)
+    {
+    vtkDICOMVR vr = query->FindDictVR(0, *tagPtr);
+    query->SetAttributeValue(*tagPtr, vtkDICOMValue(vr));
+    }
+
+  if (this->Query)
+    {
+    // add elements that the user requested for the query
+    vtkDICOMDataElementIterator iter = this->Query->Begin();
+    vtkDICOMDataElementIterator iterEnd = this->Query->End();
+    while (iter != iterEnd)
+      {
+      query->SetAttributeValue(iter->GetTag(), iter->GetValue());
+      ++iter;
+      }
+    // use a buffer size equal to one disk block
+    parser->SetBufferSize(4096);
+    }
+
+  parser->SetQuery(query);
 
   SeriesInfoList sortedFiles;
   SeriesInfoList::iterator li;
@@ -450,6 +524,15 @@ void vtkDICOMDirectory::SortFiles(vtkStringArray *input)
       return;
       }
 
+    // Check if the file matches the query
+    if (this->Query)
+      {
+      if (!parser->GetQueryMatched())
+        {
+        continue;
+        }
+      }
+
     // Insert the file into the sorted list
     FileInfo fileInfo;
     fileInfo.InstanceNumber =
