@@ -26,9 +26,12 @@
 #include <assert.h>
 
 #include <string>
+#include <vector>
+#include <algorithm>
 
 vtkStandardNewMacro(vtkDICOMParser);
 vtkCxxSetObjectMacro(vtkDICOMParser, MetaData, vtkDICOMMetaData);
+vtkCxxSetObjectMacro(vtkDICOMParser, Query, vtkDICOMMetaData);
 vtkCxxSetObjectMacro(vtkDICOMParser, Groups, vtkUnsignedShortArray);
 
 /*----------------------------------------------------------------------------
@@ -98,12 +101,19 @@ class BigEndianDecoder;
 class DecoderBase
 {
 public:
+  // Virtual destructor for completeness.
+  virtual ~DecoderBase() {}
+
   // Whether to use implicit VRs (default: explicit VRs).
   void SetImplicitVR(bool i) { this->ImplicitVR = i; }
 
   // The Item member variable is set while a sequence is decoded.
   void SetItem(vtkDICOMItem *i);
 
+  // The query with which to filter the data.
+  template<class T>
+  void SetQuery(T *q);
+
   // Read l bytes of data, or until delimiter tag found.
   // Set l to 0xffffffff to ignore length completely.
   // If the delimiter is of the form (0xgggg,0x0000), ie. if the
@@ -171,13 +181,23 @@ public:
   // for instances in the series that were already parsed.
   void HandleMissingAttributes(vtkDICOMTag tag);
 
+  // Returns true if the query contains the given tag.
+  bool QueryContains(vtkDICOMTag tag);
+
+  // Returns true if the value matches the query.
+  bool QueryMatches(const vtkDICOMValue& v);
+
+  // Returns true if the query matched.
+  bool GetQueryMatched() { return this->QueryMatched; }
+
 protected:
   // Constructor that initializes all of the members.
   DecoderBase(vtkDICOMParser *parser, vtkDICOMMetaData *data, int idx) :
     Parser(parser), Item(0), MetaData(data),
     ItemCharacterSet(vtkDICOMCharacterSet::Unknown),
     CharacterSet(vtkDICOMCharacterSet::Unknown),
-    Index(idx), ImplicitVR(0) {}
+    Index(idx), ImplicitVR(false),
+    HasQuery(false), QueryMatched(false) {}
 
   // an internal implicit little-endian decoder
   DefaultDecoder *ImplicitLE;
@@ -195,6 +215,11 @@ protected:
   int Index;
   // if this is set, then VRs are implicit
   bool ImplicitVR;
+  // the query to apply while reading the data
+  bool HasQuery;
+  bool QueryMatched;
+  vtkDICOMDataElementIterator Query;
+  vtkDICOMDataElementIterator QueryEnd;
   // this is set to the last tag that was read.
   vtkDICOMVR  LastVR;
   vtkDICOMTag LastTag;
@@ -360,6 +385,24 @@ inline void DecoderBase::SetItem(vtkDICOMItem *i)
   this->ImplicitLE->ItemCharacterSet = vtkDICOMCharacterSet::Unknown;
 }
 
+//----------------------------------------------------------------------------
+template<class T>
+void DecoderBase::SetQuery(T *q)
+{
+  if (q)
+    {
+    this->HasQuery = true;
+    this->QueryMatched = true;
+    this->Query = q->Begin();
+    this->QueryEnd = q->End();
+    }
+  else
+    {
+    this->HasQuery = false;
+    this->QueryMatched = false;
+    }
+}
+
 //----------------------------------------------------------------------------
 inline void DecoderBase::CopyBuffer(
   vtkDICOMValue *v, const unsigned char *sp, const unsigned char *cp)
@@ -578,6 +621,108 @@ void DecoderBase::HandleMissingAttributes(vtkDICOMTag tag)
   this->LastWrittenTag = tag;
 }
 
+//----------------------------------------------------------------------------
+bool DecoderBase::QueryContains(vtkDICOMTag tag)
+{
+  // if the tag isn't private
+  if ((tag.GetGroup() & 1) == 0)
+    {
+    // check to see if the tag is listed in the query
+    vtkDICOMTag lasttag;
+    while (this->Query != this->QueryEnd &&
+           (lasttag = this->Query->GetTag()) < tag)
+      {
+      ++this->Query;
+      }
+    return (lasttag == tag);
+    }
+
+  // private tags require special handling
+  unsigned short g = tag.GetGroup();
+  unsigned short e = tag.GetElement();
+
+  // first, make sure this private group is present in the query,
+  // and ignore any elements before (gggg,0010)
+  vtkDICOMTag gtag = vtkDICOMTag(g, 0x0010);
+  vtkDICOMTag ltag;
+  while (this->Query != this->QueryEnd &&
+         (ltag = this->Query->GetTag()) < gtag)
+    {
+    ++this->Query;
+    }
+  if (ltag.GetGroup() != g)
+    {
+    return false;
+    }
+
+  // if this is a creator element (e is 0x00XX), return true
+  if (e >= 0x0010 && e <= 0x00FF)
+    {
+    return true;
+    }
+
+  // else search for the creator element within the query
+  vtkDICOMDataElementIterator iter = this->Query;
+  vtkDICOMTag ctag = vtkDICOMTag(g, e >> 8);
+  const vtkDICOMValue& creator = this->MetaData->GetAttributeValue(ctag);
+  if (creator.IsValid())
+    {
+    // maximum possible creator element is (gggg,00FF)
+    gtag = vtkDICOMTag(g, 0x00FF);
+    while (iter != this->QueryEnd && iter->GetTag() <= gtag)
+      {
+      if (iter->GetValue() == creator)
+        {
+        tag = vtkDICOMTag(
+          g, (iter->GetTag().GetElement() << 8) | (e & 0x00FF));
+        break;
+        }
+      ++iter;
+      }
+    // if creator not found in query, tag obviously won't be found
+    if (iter == this->QueryEnd || iter->GetTag() > gtag)
+      {
+      return false;
+      }
+    }
+
+  // finally, look for the private tag at its resolved location
+  while (iter != this->QueryEnd && (ltag = iter->GetTag()) < tag)
+    {
+    ++iter;
+    }
+  return (ltag == tag);
+}
+
+//----------------------------------------------------------------------------
+bool DecoderBase::QueryMatches(const vtkDICOMValue& v)
+{
+  // if VR is SQ, then the matching was done when the sequence items were
+  // parsed, and we just have to return the current value of QueryMatched
+  if (v.GetVR() == vtkDICOMVR::SQ)
+    {
+    return this->QueryMatched;
+    }
+
+  // a query match is automatically defined as "true" if:
+  // 1) the element is SpecificCharacterSet, because this element exists to
+  //    describe the character encoding of the query, not because it is to
+  //    be matched (strings are converted to utf-8 prior to comparison)
+  // 2) the element is a private creator element (group # is odd, element #
+  //    is between 0x10 and 0xFF) because these exist only to identify the
+  //    creator of the private elements that appear later
+  if (this->Query->GetTag() == DC::SpecificCharacterSet ||
+      ((this->Query->GetTag().GetGroup() & 1) != 0 &&
+      this->Query->GetTag().GetElement() >= 0x0010 &&
+      this->Query->GetTag().GetElement() <= 0x00FF))
+    {
+    return true;
+    }
+
+  // else use the "Matches" method of vtkDICOMValue
+  return v.Matches(this->Query->GetValue());
+}
+
 //----------------------------------------------------------------------------
 template<>
 inline unsigned short Decoder<LE>::GetInt16(const unsigned char *ip)
@@ -869,7 +1014,8 @@ size_t Decoder<E>::ReadElementValue(
     case VTK_CHAR:
       {
       char *ptr;
-      if (vr.HasSpecificCharacterSet())
+      if (vr.HasSpecificCharacterSet() &&
+          this->LastTag > DC::SpecificCharacterSet)
         {
         ptr = v.AllocateCharData(vr, this->GetCharacterSet(), vl);
         }
@@ -960,7 +1106,48 @@ size_t Decoder<E>::ReadElementValue(
           vtkDICOMItem item(delimited, offset);
           vtkDICOMItem *olditem = this->Item;
           this->SetItem(&item);
-          this->ReadElements(cp, ep, il, endtag, l);
+
+          if (this->HasQuery)
+            {
+            assert(this->Query != this->QueryEnd);
+
+            // save the current query state before going one level deeper
+            bool hasQuery = this->HasQuery;
+            bool queryMatched = this->QueryMatched;
+            vtkDICOMDataElementIterator query = this->Query;
+            vtkDICOMDataElementIterator queryEnd = this->QueryEnd;
+
+            // set default HasQuery to 'false' to match everything
+            this->HasQuery = false;
+
+            if (query->GetValue().GetNumberOfValues() > 0)
+              {
+              // if query sequence isn't empty, set HasQuery to 'true' and
+              // use the sequence item as the new data set query
+              const vtkDICOMItem *qitems = query->GetValue().GetSequenceData();
+              assert(qitems != 0);
+              this->HasQuery = true;
+              this->QueryMatched = true;
+              this->Query = qitems[0].Begin();
+              this->QueryEnd = qitems[0].End();
+
+              // initialize queryMatched to false at the start of seq
+              queryMatched &= (seq.GetNumberOfItems() > 0);
+              }
+
+            this->ReadElements(cp, ep, il, endtag, l);
+
+            // restore the query state
+            this->HasQuery = hasQuery;
+            this->QueryMatched |= queryMatched;
+            this->Query = query;
+            this->QueryEnd = queryEnd;
+            }
+          else
+            {
+            // if HasQuery is false, simply read the item
+            this->ReadElements(cp, ep, il, endtag, l);
+            }
           seq.AddItem(item);
           this->SetItem(olditem);
           }
@@ -1040,6 +1227,43 @@ bool Decoder<E>::ReadElements(
     // break if delimiter found
     if (!readGroup && tag == delimiter) { break; }
 
+    // skip the value if this tag is not in the query
+    if (this->HasQuery && !this->QueryContains(tag))
+      {
+      if (vl != HxFFFFFFFF)
+        {
+        // constant length item
+        tl = this->SkipData(cp, ep, vl);
+        if (tl != static_cast<size_t>(vl)) { return false; }
+        }
+      else
+        {
+        // if vl == 0xffffffff, the value is delimited
+        vtkDICOMTag newdelim(HxFFFE, HxE0DD);
+        if (tag == vtkDICOMTag(HxFFFE, HxE000))
+          { // if tag is item tag, use item delimiter
+          newdelim = vtkDICOMTag(HxFFFE, HxE00D);
+          }
+        if (vr != vtkDICOMVR::UN)
+          {
+          // Value is either a sequence or is encapsulated data
+          if (!this->SkipElements(cp, ep, vl, newdelim, NULL))
+            {
+            return false;
+            }
+          }
+        else
+          {
+          // VR of UN indicates the value is an implicit LE sequence
+          if (!this->ImplicitLE->SkipElements(cp, ep, vl, newdelim, NULL))
+            {
+            return false;
+            }
+          }
+        }
+      continue;
+      }
+
     // read the value
     vtkDICOMValue v;
     size_t rl = 0;
@@ -1097,6 +1321,18 @@ bool Decoder<E>::ReadElements(
       }
     cout << "\n";
     */
+
+    // check if the value matches the query, a mismatched value will
+    // be the final value that is read by the parser
+    if (this->HasQuery && !this->QueryMatches(v))
+      {
+      this->QueryMatched = false;
+      // break only if the query fails at the root level
+      if (this->Item == 0)
+        {
+        break;
+        }
+      }
     }
 
   bytesRead += tl;
@@ -1255,6 +1491,8 @@ vtkDICOMParser::vtkDICOMParser()
 {
   this->FileName = NULL;
   this->MetaData = NULL;
+  this->Query = NULL;
+  this->QueryItem = NULL;
   this->Groups = NULL;
   this->InputFile = NULL;
   this->BytesRead = 0;
@@ -1265,6 +1503,7 @@ vtkDICOMParser::vtkDICOMParser()
   this->ChunkSize = 0;
   this->Index = -1;
   this->PixelDataFound = false;
+  this->QueryMatched = false;
   this->ErrorCode = 0;
 }
 
@@ -1272,17 +1511,38 @@ vtkDICOMParser::vtkDICOMParser()
 vtkDICOMParser::~vtkDICOMParser()
 {
   delete [] this->FileName;
+  delete this->QueryItem;
 
   if (this->MetaData)
     {
     this->MetaData->Delete();
     }
+  if (this->Query)
+    {
+    this->Query->Delete();
+    }
   if (this->Groups)
     {
     this->Groups->Delete();
     }
 }
 
+//----------------------------------------------------------------------------
+void vtkDICOMParser::SetQueryItem(const vtkDICOMItem& query)
+{
+  if (this->Query)
+    {
+    this->Query->Delete();
+    this->Query = 0;
+    }
+  delete this->QueryItem;
+  this->QueryItem = 0;
+  if (query.GetNumberOfDataElements() > 0)
+    {
+    this->QueryItem = new vtkDICOMItem(query);
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkDICOMParser::SetBufferSize(int size)
 {
@@ -1321,9 +1581,18 @@ bool vtkDICOMParser::ReadFile(vtkDICOMMetaData *data, int idx)
 {
   // Mark pixel data as not found yet
   this->PixelDataFound = false;
+  this->QueryMatched = (this->Query != 0);
   this->FileOffset = 0;
   this->FileSize = 0;
 
+  // Query cannot be used with indexing
+  if (idx > 0 && this->Query)
+    {
+    this->SetErrorCode(vtkErrorCode::UnknownError);
+    vtkErrorMacro("ReadFile: Querying cannot be used when Index > 0");
+    return false;
+    }
+
   // Check that the file name has been set.
   if (!this->FileName)
     {
@@ -1479,12 +1748,58 @@ bool vtkDICOMParser::ReadMetaData(
     decoder = &decoderBE;
     }
 
-  vtkUnsignedShortArray *groups = this->Groups;
+  // get the Query
+  vtkDICOMDataElementIterator iter;
+  vtkDICOMDataElementIterator iterEnd;
+  bool hasQuery = false;
+  if (this->Query)
+    {
+    hasQuery = true;
+    iter = this->Query->Begin();
+    iterEnd = this->Query->End();
+    decoder->SetQuery(this->Query);
+    }
+  else if (this->QueryItem)
+    {
+    hasQuery = true;
+    iter = this->QueryItem->Begin();
+    iterEnd = this->QueryItem->End();
+    decoder->SetQuery(this->QueryItem);
+    }
+
+  // make a list of the groups of interest
+  std::vector<unsigned short> groups;
+  if (hasQuery)
+    {
+    unsigned short lastg = 0;
+    while (iter != iterEnd)
+      {
+      unsigned short g = iter->GetTag().GetGroup();
+      if (g > lastg)
+        {
+        lastg = g;
+        groups.push_back(g);
+        }
+      ++iter;
+      }
+    }
+  else if (this->Groups)
+    {
+    vtkIdType n = this->Groups->GetNumberOfTuples();
+    for (vtkIdType i = 0; i < n; i++)
+      {
+      groups.push_back(this->Groups->GetValue(i));
+      }
+    std::sort(groups.begin(),groups.end());
+    groups.erase(std::unique(groups.begin(),groups.end()), groups.end());
+    }
+  std::vector<unsigned short>::iterator giter = groups.begin();
 
   // read group-by-group
   bool foundPixelData = false;
   bool readFailure = false;
-  while (!foundPixelData && !readFailure)
+  bool queryFailure = false;
+  while (!foundPixelData && !readFailure && !queryFailure)
     {
     unsigned int g = decoder->PeekGroup(cp, ep);
 
@@ -1493,14 +1808,13 @@ bool vtkDICOMParser::ReadMetaData(
 
     // do we want to read or skip this group?
     bool found = true;
-    if (groups)
+    if (groups.size() > 0)
       {
-      found = false;
-      vtkIdType n = groups->GetNumberOfTuples();
-      for (vtkIdType i = 0; i < n && !found; i++)
+      while (giter != groups.end() && *giter < g)
         {
-        found = (g == groups->GetValue(i));
+        ++giter;
         }
+      found = (giter != groups.end() && *giter == g);
       }
 
     // create a delimiter to read/skip only this group
@@ -1517,6 +1831,7 @@ bool vtkDICOMParser::ReadMetaData(
     if (found && meta)
       {
       readFailure = !decoder->ReadElements(cp, ep, HxFFFFFFFF, delimiter);
+      queryFailure = (hasQuery && !decoder->GetQueryMatched());
       }
     else
       {
@@ -1524,6 +1839,7 @@ bool vtkDICOMParser::ReadMetaData(
       }
     }
 
+  this->QueryMatched = decoder->GetQueryMatched();
   this->PixelDataFound = (decoder->GetLastTag() == DC::PixelData);
   if (meta && this->PixelDataFound)
     {
@@ -1632,5 +1948,8 @@ void vtkDICOMParser::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "MetaData: " << this->MetaData << "\n";
   os << indent << "Index: " << this->Index << "\n";
   os << indent << "BufferSize: " << this->BufferSize << "\n";
+  os << indent << "Query: " << this->Query << "\n";
+  os << indent << "QueryMatched: "
+     << (this->QueryMatched ? "True\n" : "False\n");
   os << indent << "Groups: " << this->Groups << "\n";
 }
