@@ -375,7 +375,7 @@ const vtkDICOMValue& vtkDICOMReaderGetFrameAttributeValue(
 // compute spatial location from position and orientation
 double vtkDICOMReaderComputeLocation(
   const vtkDICOMValue& pv, const vtkDICOMValue& ov,
-  double checkOrientation[10], bool *checkPtr)
+  double checkOrientation[14], bool *checkPtr)
 {
   double location = 0.0;
 
@@ -402,6 +402,7 @@ double vtkDICOMReaderComputeLocation(
         checkOrientation[i] = orientation[i];
         checkOrientation[3+i] = orientation[3+i];
         checkOrientation[6+i] = position[i];
+        checkOrientation[10+i] = normal[i];
         }
       // indicate that checkOrientation has been set
       checkOrientation[9] = 1.0;
@@ -438,11 +439,28 @@ double vtkDICOMReaderComputeLocation(
       v[0] = position[0] - checkPosition[0];
       v[1] = position[1] - checkPosition[1];
       v[2] = position[2] - checkPosition[2];
-      double t = vtkMath::Dot(v, normal);
-      v[0] -= t*normal[0];
-      v[1] -= t*normal[1];
-      v[2] -= t*normal[2];
-      double d = v[0]*v[0] + v[1]*v[1] + v[2]*v[2];
+      double vdir = (vtkMath::Dot(v, normal) < 0 ? -1.0 : 1.0);
+
+      // set the check vector if it isn't set yet
+      if (checkOrientation[13] == 0 &&
+          v[0]*v[0] + v[1]*v[1] + v[2]*v[2] >
+          positionTolerance*positionTolerance)
+        {
+        for (int i = 0; i < 3; i++)
+          {
+          checkOrientation[10+i] = vdir*v[i];
+          }
+        vtkMath::Normalize(&checkOrientation[10]);
+        }
+
+      // compare vector to check vector
+      double w = vtkMath::Norm(&checkOrientation[10]);
+      double t = vtkMath::Dot(v, &checkOrientation[10])/w;
+      double dv[3];
+      dv[0] = v[0] - t*checkOrientation[10];
+      dv[1] = v[1] - t*checkOrientation[11];
+      dv[2] = v[2] - t*checkOrientation[12];
+      double d = dv[0]*dv[0] + dv[1]*dv[1] + dv[2]*dv[2];
       // the position tolerance is in millimetres
       if (d > (positionTolerance*positionTolerance +
                t*t*directionTolerance*directionTolerance))
@@ -450,6 +468,14 @@ double vtkDICOMReaderComputeLocation(
         // positions don't line up along the normal
         *checkPtr = false;
         }
+      else if (checkOrientation[13] != 0 && vtkMath::Norm(v) > w)
+        {
+        // use new vector as check vector if it is longer
+        for (int i = 0; i < 3; i++)
+          {
+          checkOrientation[10+i] = vdir*v[i];
+          }
+        }
       }
     if (*checkPtr == false)
       {
@@ -459,13 +485,79 @@ double vtkDICOMReaderComputeLocation(
         checkOrientation[i] = orientation[i];
         checkOrientation[3+i] = orientation[3+i];
         checkOrientation[6+i] = position[i];
+        checkOrientation[10+i] = normal[i];
         }
+      // mark the check vector as unset
+      checkOrientation[13] = 0;
       }
     }
 
   return location;
 }
 
+// compute the spacing between slices
+double vtkDICOMReaderComputeSpacing(
+  const std::vector<vtkDICOMReaderSortInfo>& info,
+  int *missing)
+{
+  double a = 0;
+  double v = VTK_DOUBLE_MAX;
+  size_t m = VTK_UNSIGNED_INT_MAX;
+  int o = 0;
+
+  while (m != 0)
+    {
+    // compute the mean spacing and the variance in spacing
+    double s1 = 0.0;
+    double s2 = 0.0;
+    size_t n = 0;
+    o = 0;
+    for (size_t i = 1; i < info.size(); i++)
+      {
+      // first verify that locations aren't identical
+      if (vtkDICOMReaderSortInfo::CompareLocation(info[i-1], info[i]))
+        {
+        // compute the offset
+        double d = info[i].ComputedLocation - info[i-1].ComputedLocation;
+        double dd = (d - a)*(d - a);
+        // make sure offset is within 3 sigma or 10% of average offset
+        if (dd < 0.01*a*a || dd < 9*v)
+          {
+          s1 += d;
+          s2 += d*d;
+          n++;
+          }
+        else
+          {
+          o++;
+          }
+        }
+      }
+
+    // no results!
+    if (n == 0)
+      {
+      break;
+      }
+
+    // compute the mean and variance
+    a = s1 / n;
+    v = s2/n - a*a;
+
+    // break if within tolerance or if results haven't changed
+    if (v/(a*a) < 1e-6 || n == m)
+      {
+      break;
+      }
+
+    // save count to use as check
+    m = n;
+    }
+
+  *missing = o;
+  return a;
+}
+
 // a simple struct to provide info for each frame to be read
 struct vtkDICOMReaderFrameInfo
 {
@@ -558,7 +650,7 @@ void vtkDICOMReader::SortFiles(vtkIntArray *files, vtkIntArray *frames)
 
   // important position-related variables
   std::vector<size_t> volumeBreaks;
-  double checkOrientation[10] = {};
+  double checkOrientation[14] = {};
   bool canSortByLocation = true;
   double spacingBetweenSlices =
     meta->GetAttributeValue(DC::SpacingBetweenSlices).AsDouble();
@@ -583,7 +675,7 @@ void vtkDICOMReader::SortFiles(vtkIntArray *files, vtkIntArray *frames)
     vtkDICOMValue firstStackId;
     vtkDICOMValue desiredStackId =
       vtkDICOMValue(vtkDICOMVR::SH, this->DesiredStackID);
-    double stackCheckNormal[10] = {};
+    double stackCheckNormal[14] = {};
     bool canSortStackByIPP = true;
 
     // files have enhanced frame information
@@ -1042,9 +1134,18 @@ void vtkDICOMReader::SortFiles(vtkIntArray *files, vtkIntArray *frames)
     double locDiff = 0;
     if (locations > 1)
       {
-      double firstLocation = info.front().ComputedLocation;
-      double finalLocation = info.back().ComputedLocation;
-      locDiff = (finalLocation - firstLocation)/(locations - 1);
+      int missing = 0;
+      locDiff = vtkDICOMReaderComputeSpacing(info, &missing);
+      if (missing > 0)
+        {
+        vtkWarningMacro("Series has " << missing <<
+                        " gaps due to missing slices.");
+        }
+      if (locations > spatialLocations)
+        {
+        // squeeze time slices between spatial slices
+        locDiff *= spatialLocations*1.0/locations;
+        }
       }
     if (locDiff > 0)
       {
@@ -1217,7 +1318,7 @@ int vtkDICOMReader::RequestInformation(
 
   // Get the file and frame for the first slice
   int fileIndex = this->FileIndexArray->GetComponent(0, 0);
-  int frameIndex = this->FileIndexArray->GetComponent(0, 0);
+  int frameIndex = this->FrameIndexArray->GetComponent(0, 0);
 
   // image dimensions
   int columns = this->MetaData->GetAttributeValue(DC::Columns).AsInt();
@@ -1454,36 +1555,116 @@ int vtkDICOMReader::RequestInformation(
   // the user with a 4x4 matrix that can transform VTK's data coordinates
   // into DICOM's patient coordinates, as defined in the DICOM standard
   // Part 3 Appendix C 7.6.2 "Image Plane Module".
-  vtkDICOMValue pv = this->MetaData->GetAttributeValue(
-    fileIndex, frameIndex, DC::ImagePositionPatient);
-  vtkDICOMValue ov = this->MetaData->GetAttributeValue(
-    fileIndex, frameIndex, DC::ImageOrientationPatient);
-  if (pv.GetNumberOfValues() == 3 && ov.GetNumberOfValues() == 6)
+
+  vtkIdType numSlices = this->FileIndexArray->GetNumberOfTuples();
+  std::vector<double> points;
+  double centroid[3] = { 0.0, 0.0, 0.0 };
+  double orient[6] = { 1.0, 0.0, 0.0, 0.0, 1.0, 0.0 };
+  double point[3] = { 0.0, 0.0, 0.0 };
+  double normal[3] = { 0.0, 0.0, 1.0 };
+
+  // go through the slices in reverse order, so we end on the first
+  for (vtkIdType iSlice = numSlices-1; iSlice >= 0; --iSlice)
     {
-    double orient[6], normal[3], point[3];
-    pv.GetValues(point, 3);
-    ov.GetValues(orient, 6);
-
-    if (this->MemoryRowOrder == vtkDICOMReader::BottomUp)
+    int iFile = this->FileIndexArray->GetComponent(iSlice, 0);
+    int iFrame = this->FrameIndexArray->GetComponent(iSlice, 0);
+    vtkDICOMValue pv = this->MetaData->GetAttributeValue(
+      iFile, iFrame, DC::ImagePositionPatient);
+    vtkDICOMValue ov = this->MetaData->GetAttributeValue(
+      fileIndex, frameIndex, DC::ImageOrientationPatient);
+    if (pv.GetNumberOfValues() == 3 && ov.GetNumberOfValues() == 6)
       {
-      // calculate position of point at lower left
-      double yspacing = this->DataSpacing[1];
-      point[0] = point[0] + orient[3]*yspacing*(rows - 1);
-      point[1] = point[1] + orient[4]*yspacing*(rows - 1);
-      point[2] = point[2] + orient[5]*yspacing*(rows - 1);
+      pv.GetValues(point, 3);
+      ov.GetValues(orient, 6);
+      vtkMath::Normalize(&orient[0]);
+      vtkMath::Normalize(&orient[3]);
+      vtkMath::Cross(&orient[0], &orient[3], normal);
+      vtkMath::Normalize(normal);
+      // re-orthogonalize x vector (improve precision)
+      vtkMath::Cross(&orient[3], normal, &orient[0]);
 
-      // measure orientation from lower left corner upwards
-      orient[3] = -orient[3];
-      orient[4] = -orient[4];
-      orient[5] = -orient[5];
+      if (this->MemoryRowOrder == vtkDICOMReader::BottomUp)
+        {
+        // calculate position of point at lower left
+        double yspacing = this->DataSpacing[1];
+        point[0] = point[0] + orient[3]*yspacing*(rows - 1);
+        point[1] = point[1] + orient[4]*yspacing*(rows - 1);
+        point[2] = point[2] + orient[5]*yspacing*(rows - 1);
+
+        orient[3] = -orient[3];
+        orient[4] = -orient[4];
+        orient[5] = -orient[5];
+        }
+
+      size_t ip = points.size();
+      points.resize(ip + 3);
+      for (int ii = 0; ii < 3; ii++)
+        {
+        centroid[ii] += point[ii];
+        points[ip+ii] = point[ii];
+        }
+      }
+    }
+
+  // compute Z vector by fitting points to a line
+  double vector[3] = { normal[0], normal[1], normal[2] };
+  size_t nPoints = points.size()/3;
+  if (nPoints > 0)
+    {
+    centroid[0] /= nPoints;
+    centroid[1] /= nPoints;
+    centroid[2] /= nPoints;
+
+    // use Jacobi to compute line of best fit
+    double storage[18] = {};
+    double *A[3] = { &storage[0], &storage[3], &storage[6] };
+    double *E[3] = { &storage[9], &storage[12], &storage[15] };
+    for (size_t iPoint = 0; iPoint < nPoints; iPoint++)
+      {
+      for (int ii = 0; ii < 3; ii++)
+        {
+        for (int jj = 0; jj < 3; jj++)
+          {
+          A[ii][jj] = ((points[3*iPoint + ii] - centroid[ii]) *
+                       (points[3*iPoint + jj] - centroid[jj]));
+          }
+        }
+      }
+    double eigenvalues[3];
+    vtkMath::Jacobi(A, eigenvalues, E);
+
+    // only use eigenvector if the points fit a line very precisely
+    if (eigenvalues[1]*eigenvalues[1] + eigenvalues[2]*eigenvalues[2] <
+        1e-6*eigenvalues[0]*eigenvalues[0])
+      {
+      // create the vector, dot(vector,normal) should be unity
+      double vdn = E[0][0]*normal[0] + E[1][0]*normal[1] + E[2][0]*normal[2];
+      if (vdn > 0)
+        {
+        vector[0] = E[0][0]/vdn;
+        vector[1] = E[1][0]/vdn;
+        vector[2] = E[2][0]/vdn;
+
+        // check difference between this vector and the normal
+        double vcn[3];
+        vtkMath::Cross(vector, normal, vcn);
+        if (vtkMath::Norm(vcn) < 1e-4)
+          {
+          // use normal (more precise) if they are the same
+          vector[0] = normal[0];
+          vector[1] = normal[1];
+          vector[2] = normal[2];
+          }
+        }
       }
 
-    vtkMath::Cross(&orient[0], &orient[3], normal);
+    // build the patient matrix
     double pm[16];
-    pm[0] = orient[0]; pm[1] = orient[3]; pm[2] = normal[0]; pm[3] = point[0];
-    pm[4] = orient[1]; pm[5] = orient[4]; pm[6] = normal[1]; pm[7] = point[1];
-    pm[8] = orient[2]; pm[9] = orient[5]; pm[10] = normal[2]; pm[11] = point[2];
+    pm[0] = orient[0]; pm[1] = orient[3]; pm[2] = vector[0]; pm[3] = point[0];
+    pm[4] = orient[1]; pm[5] = orient[4]; pm[6] = vector[1]; pm[7] = point[1];
+    pm[8] = orient[2]; pm[9] = orient[5]; pm[10] = vector[2]; pm[11] = point[2];
     pm[12] = 0.0; pm[13] = 0.0; pm[14] = 0.0; pm[15] = 1.0;
+
     this->PatientMatrix->DeepCopy(pm);
     }
   else
